<?php
/** 
  * Czech translation
  * 
  * @version 0.4
  * @author Vladimír Mach
  * @mail wladik@gmail.com
  * 
  */
$labels = array();
$labels['verifiedsender'] = 'Platný podpis z domény odesílatele. Ověřil: ';
$labels['invalidsignature'] = 'Neplatný podpis! Ověřil: ';
$labels['nosignature'] = 'Podpis není k dospozici.';
$labels['thirdpartysig'] = 'Podpis odesílatele není k dispozici, ale je tu podpis třetí strany. Ověřil: ';

?>
