<?php

$labels = array();
$labels['verifiedsender'] = '送信者のドメインによる正当な署名です。検証内容： ';
$labels['invalidsignature'] = '不正な署名です！ 検証内容： ';
$labels['nosignature'] = '署名されていません';
$labels['thirdpartysig'] = '送信者のドメインによる署名はありませんが、第三者による署名があります。検証内容： ';

?>
